"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NEG_ONE_I80F48 = exports.ZERO_I80F48 = exports.ONE_I80F48 = exports.I80F48 = void 0;
const bn_js_1 = __importDefault(require("bn.js"));
const big_js_1 = __importDefault(require("big.js"));
// TODO - this whole class is inefficient; consider optimizing
class I80F48 {
    constructor(data) {
        // TODO why is this calculated here? should just be static part of the class
        this.maxValue = new bn_js_1.default(2)
            .pow(new bn_js_1.default(I80F48.MAX_SIZE))
            .div(new bn_js_1.default(2))
            .sub(new bn_js_1.default(1));
        this.minValue = new bn_js_1.default(2).pow(new bn_js_1.default(I80F48.MAX_SIZE)).div(new bn_js_1.default(2)).neg();
        if (data.lt(this.minValue) || data.gt(this.maxValue)) {
            throw new Error('Number out of range');
        }
        this.data = data;
        this.binaryLayout = data
            .toTwos(I80F48.MAX_SIZE)
            .toString(2, I80F48.MAX_SIZE)
            .replace(/-/g, '');
    }
    static fromNumber(x) {
        return this.fromString(x.toString());
    }
    static fromString(x) {
        const multiplier = new big_js_1.default(2).pow(this.FRACTIONS);
        const initialValue = new big_js_1.default(x).times(multiplier);
        const fixedPointValue = new bn_js_1.default(initialValue.round().toFixed());
        return new I80F48(fixedPointValue);
    }
    static fromI64(x) {
        return this.fromString(x.toString());
    }
    static fromU64(x) {
        return this.fromString(x.toString());
    }
    toString() {
        return this.toFixed();
    }
    toFixed(decimals) {
        return this.toBig().toFixed(decimals);
    }
    toBig() {
        const divider = new big_js_1.default(2).pow(I80F48.FRACTIONS);
        return new big_js_1.default(this.data.toString()).div(divider);
    }
    toNumber() {
        return this.toBig().toNumber();
    }
    static fromArray(src) {
        if (src.length !== 16) {
            throw new Error('Uint8Array must be of length 16');
        }
        return new I80F48(new bn_js_1.default(src, 'le').fromTwos(I80F48.MAX_SIZE));
    }
    toArray() {
        return new Uint8Array(this.data.toTwos(I80F48.MAX_SIZE).toArray('le', 16));
    }
    toArrayLike(ArrayType, endian, length) {
        return this.data
            .toTwos(I80F48.MAX_SIZE)
            .toArrayLike(ArrayType, endian, length);
    }
    getInternalValue() {
        return this.data;
    }
    getBinaryLayout() {
        return this.binaryLayout;
    }
    add(x) {
        return new I80F48(this.data.add(x.getInternalValue()));
    }
    sub(x) {
        return new I80F48(this.data.sub(x.getInternalValue()));
    }
    /**
     * Multiply the two and shift
     * @param x
     */
    mul(x) {
        const divider = new big_js_1.default(2).pow(I80F48.FRACTIONS);
        const result = new big_js_1.default(this.data.mul(x.getInternalValue()).toString())
            .div(divider)
            .round()
            .toFixed();
        return new I80F48(new bn_js_1.default(result));
    }
    div(x) {
        const multiplier = new big_js_1.default(2).pow(I80F48.FRACTIONS);
        const result = new big_js_1.default(this.data.toString())
            .div(x.getInternalValue().toString())
            .times(multiplier)
            .round()
            .toFixed();
        return new I80F48(new bn_js_1.default(result));
    }
    gt(x) {
        return this.data.gt(x.getInternalValue());
    }
    lt(x) {
        return this.data.lt(x.getInternalValue());
    }
    gte(x) {
        return this.data.gte(x.getInternalValue());
    }
    lte(x) {
        return this.data.lte(x.getInternalValue());
    }
    eq(x) {
        // TODO make sure this works when they're diff signs or 0
        return this.data.eq(x.getInternalValue());
    }
    cmp(x) {
        // TODO make sure this works when they're diff signs or 0
        return this.data.cmp(x.getInternalValue());
    }
    neg() {
        return this.mul(exports.NEG_ONE_I80F48);
    }
    isNeg() {
        return this.data.isNeg();
    }
}
exports.I80F48 = I80F48;
/**
This is represented by a 128 bit signed integer underneath
The first 80 bits are treated as an integer and last 48 bits are treated as fractional part after binary point
It's possible to think of an I80F48 as an i128 divided by 2 ^ 40

Read up on how fixed point math works: https://inst.eecs.berkeley.edu/~cs61c/sp06/handout/fixedpt.html
Read up on how 2s complement works: https://en.wikipedia.org/wiki/Two%27s_complement
 */
I80F48.MAX_SIZE = 128;
I80F48.FRACTIONS = 48;
exports.ONE_I80F48 = I80F48.fromString('1');
exports.ZERO_I80F48 = I80F48.fromString('0');
exports.NEG_ONE_I80F48 = I80F48.fromString('-1');
//# sourceMappingURL=fixednum.js.map